;------------------------------------------------------------------------------
;
;  NSSDC/CDF				    IDL/CDF Interface, CDFattrInquire.
;
;  Version 1.4, 21-Aug-96, Hughes STX.
;
;  Modification history:
;
;   V1.0  21-Sep-92, H Leckner	Original version.
;   V1.1  10-Dec-92, H Leckner	Removed print/help debug statements.
;   V1.2  13-May-93, J Love	CDF V2.4 (`idl_cdf.pro' split into separate
;                               files as required for online help in IDL).
;   V1.3   1-Nov-94, J Love	CDF V2.5.
;   V1.3a 26-Jun-95, J Love	IDL 4.0.
;   V1.4  21-Aug-96, J Love	CDF V2.6.
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
; CDFattrInquire.
;------------------------------------------------------------------------------

;+
; NAME:
;       CDFattrInquire
;
; PURPOSE:
;       `CDFattrInquire' is used to inquire about an attribute in a CDF.
;
;       This function is part of the CDF interface provided with the CDF
;       distribution.  IDL also provides its own built-in interface to CDFs.
;
; CALLING SEQUENCE:
;       status = CDFattrInquire (id, attrNum, attrName, attrScope, maxEntry)
;
; INPUTS:
;       id:             LONG.  Identifier of the CDF being accessed.  This
;                       identifier must have been returned from a successful
;                       call to `CDFcreate' or `CDFopen'.
;       attrNum:        LONG.  The number of the attribute.
;
;       All input variables must have been created/initialized before calling
;       `CDFattrInquire'.
;
; OUTPUTS:
;       attrName:       STRING.  The attribute name.
;       attrScope:      LONG.  The attribute's scope.  This will be one of
;                       attribute scopes defined by `cdf0x.pro'.
;       maxEntry:       LONG.  The maximum entry number written for this
;                       attribute.
;
;       status:         LONG.  A completion status code.  CDF_OK indicates
;                       success.  The possible status codes are defined by
;                       `cdf0x.pro'.
;
;       All output variables are (re)created/assigned by `CDFattrInquire'.
;
; EXAMPLE:
;       IDL> @cdf0x.pro
;       IDL> status = CDFattrInquire (id, CDFattrNum(id,'TITLE'), attrName, $
;       IDL>                          attrScope, maxEntry)
;       IDL> if (status lt CDFx.CDF_WARN) print, 'CDFattrInquire failed.'
;
; RESTRICTIONS:
;       None.
;
; REVISION HISTORY:
;       13-May-93       Original version.
;        1-Nov-94       CDF V2.5.
;       26-Jun-95       IDL 4.0.
;       21-Aug-96       CDF V2.6.
;-

function CDFattrInquire, CDF_id,attr_num,attr_mnemonic,attr_scope,max_entry
common cdfcurrent_, cdfid_, cdfstatus_
on_error, 1
idl_cdfattrinq, cdfid_, cdfstatus_, status, $
		CDF_id, attr_num, attr_mnemonic, attr_scope, max_entry
return, status
end
